/*
 * open.c
 *
 * Copyright (C) 1993 Alain Knaff
 */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

#define __open oldopen
#include <fcntl.h>
#undef __open

#include <sys/wait.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <string.h>
#include "zlibc.h"
#include <syscall.h>
#include <time.h>
#include <sys/time.h>
#include <utime.h>


#ifdef NEED_PROTOTYPES
int open(__const char *name, int flags, mode_t mode)
#else
int open(name,flags,mode)
char *name; int flags; mode_t mode;
#endif
{
  int fd;
  int zfd;
  int pid;
  union wait status;
  struct stat buf;
#ifdef linux
  struct utimbuf buf2;
#else
  struct timeval buf2[2];
#endif

  int pipeout[2];

  char newname[MAXPATHLEN + MAXEXTLEN + 1];
  char pname[MAXPATHLEN + MAXEXTLEN + 1];

  fd = syscall(SYS_open,name,flags,mode);

#ifdef DEBUG
  fprintf(stderr,"file opened *: %s %d\n", name,fd);
#endif

  /* success to open uncompressed file */
  if ( fd >= 0 || errno != ENOENT )
    return fd;

  /* only readonly files may be opened in compressed mode */
  if ( flags & O_ACCMODE != O_RDONLY )
    return fd;

  zlib_initialise();

  if ( zlib_mode & CM_DISAB )
    return fd;

  if ( zlib_mode & CM_VERBOSE )
    fprintf(stderr,"opening %s\n", name);

  strncpy(newname,name,1024);
  strcat(newname,zlib_ext);

  /* open compressed file */

  zfd = syscall(SYS_open,newname,flags,mode);

  if ( zfd < 0 )
    return zfd;

  if ( zlib_getfiletype(name) >= PM_HIDE_PIPE ){

    /* make a pipe */
    if(pipe(pipeout) < 0 )
      return -1;
    
    /* double fork */
    switch(pid=fork()){
    case 0: /* son */
      switch(fork()){
      case 0: /* son of son */
	if ( zfd != 0 ){
	  close(0);
	  dup(zfd); /* compressed file, input */
	}
	
	if ( pipeout[1] != 1 ){
	  close(1);
	  dup(pipeout[1]);
	}

	close(pipeout[0]);
	close(pipeout[1]);

	execvp ( zlib_uncompressor[0], zlib_uncompressor );
	
	if ( zlib_mode & CM_VERBOSE ){
	  perror("exec uncompressor");
	  exit(1);
	}
      case -1: /* error */
	if ( zlib_mode & CM_VERBOSE )
	  perror("fork error");
	exit(1);
      default: /* father */
	exit(0); /* son of son will be taken over by init */
      }
    case -1: /* error */
      return -1;
    }
    close(pipeout[1]); /* close write end of pipe */


    close(zfd);        /* close raw compressed file */
    fd=dup(pipeout[0]);
    close(pipeout[0]);
    wait4(pid,&status,0,0); /* wait for son */
    
  } else {

    sprintf(pname,"%s/pipe.%d",zlib_tmp,getpid());

    syscall(SYS_unlink,pname);
    pipeout[0]=syscall(SYS_open,pname, O_RDWR | O_CREAT,0400);     
    /* temporary output file */
    if ( pipeout[0] < 0 ){
      if ( zlib_mode & CM_VERBOSE )
	perror("could not create uncompressed file");
      return -1;
    }

    switch(pid=fork()){
    case 0: /* son  */
      if ( zfd != 0 ){
	close(0);
	dup(zfd); /* compressed file, input */
      }

      if ( pipeout[0] != 1 ){
	close(1);
	dup(pipeout[0]);
      }

      execvp ( zlib_uncompressor[0], zlib_uncompressor ) ;
      if ( zlib_mode & CM_VERBOSE )
	perror("exec uncompressor");
      exit(1);
    case -1: /* error */
      return -1;
    }

    wait4(pid,&status,0,0); /* wait for son */
       
    /* touch it */
    if ( syscall(SYS_fstat, zfd, &buf) >= 0 ){
      close(pipeout[0]);
#ifdef linux
      buf2.actime = buf.st_atime;
      buf2.modtime = buf.st_mtime;
      syscall(SYS_utime,pname, &buf2);
#else
      buf2[0].tv_sec =  buf.st_atime;
      buf2[0].tv_usec = 0;
      buf2[1].tv_sec =  buf.st_mtime;
      buf2[1].tv_usec = 0;
      syscall(SYS_utimes,pname, buf2);
#endif
    }
    
    close(zfd);        /* close raw compressed file */
    close(pipeout[0]);
    
    /* re-open it */
    fd=syscall(SYS_open,pname, O_RDONLY);
    
    syscall(SYS_unlink,pname);  
  }
  return fd;
}



