/*
 * stat.c
 *
 * Copyright (C) 1993 Alain Knaff
 */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <string.h>
#include "zlibc.h"
#include <syscall.h>

#ifdef NEED_PROTOTYPES
static int zstat(__const char *name, char *newname, struct stat *buf)
#else
static int zstat(name,newname,buf)
char *name; char *newname; struct stat *buf;
#endif
{
  int fd,st;
  unsigned char size_buffer[4];

  if( zlib_getfiletype(name) == PM_LEAVE_COMPR ){
    errno = ENOENT ;
    return -1;
  }

  fd=syscall(SYS_open,newname, O_RDONLY);
  if ( fd < 0 ){
    if ( errno == ENOENT )
      return -1;
    return 2;
  } else {  
    st = fstat(fd,buf);    
    if ( st < 0 ){
      close(fd);
      return st;
    }	
    lseek(fd, -4 , SEEK_END);
    read(fd, size_buffer, 4 );

    buf->st_size = size_buffer[0] + ( size_buffer[1] << 8 ) + 
                  (size_buffer[2] << 16 ) + (size_buffer[3] << 24 );
    close(fd);
    
    buf->st_mode &= 0444;
    switch ( zlib_getfiletype(name) ){
    case PM_SHOW_PIPE:
      buf->st_mode |= S_IFIFO;
      break;
    case PM_HIDE_PIPE:
    case PM_USE_TMP_FILE:
      buf->st_mode |= S_IFREG;
      break;
    }

#ifdef NEGATE_INODES
    buf->st_ino= -buf->st_ino;
#endif
  }
  return 0;
}

#ifdef NEED_PROTOTYPES
#ifdef linux
int __stat(const char *file_name, struct stat *buf)
#else
int stat(const char *file_name, struct stat *buf)
#endif
#else
int stat(file_name,buf)
char *file_name; struct stat *buf;
#endif
{
  int st;

  errno = 0;

  st = syscall(SYS_stat,file_name, buf );

  if ( st >= 0 || errno != ENOENT )
    return st;

  {
    char newname[MAXPATHLEN + MAXEXTLEN + 1];

    zlib_initialise();
    if ( zlib_mode & CM_DISAB )
      return st;

    if ( zlib_mode & CM_VERBOSE )
      fprintf(stderr,"stating %s\n", file_name);

    strncpy(newname,file_name,MAXPATHLEN);
    strcat(newname,zlib_ext);

    st = zstat(file_name,newname, buf);
    if ( st == 2 ){
      st = stat(newname, buf);
      buf->st_size = 0;
    }

    if ( st < 0 )
      return st;

  }
  return st;
}

#ifdef NEED_PROTOTYPES
int lstat(const char *file_name, struct stat *buf)
#else
int lstat(file_name,buf)
char *file_name; struct stat *buf;
#endif
{
  int st;

  st = syscall(SYS_lstat,file_name, buf );

  if ( st >= 0 || errno != ENOENT )
    return st;

  {
    char newname[MAXPATHLEN + MAXEXTLEN + 1];

    zlib_initialise();
    if ( zlib_mode & CM_DISAB )
      return st;
    if ( zlib_mode & CM_VERBOSE )
      fprintf(stderr,"lstating %s\n", file_name);

    strncpy(newname,file_name,MAXPATHLEN);
    strcat(newname,zlib_ext);

    st = syscall(SYS_lstat,newname, buf);

    if ( st < 0 )
      return st;

    if ( S_ISLNK(buf->st_mode) ){
      if ( buf->st_size > zlib_extlen )
	buf->st_size -= zlib_extlen;
      return st;
    }
     
    st = zstat(file_name,newname,buf);
    if ( st == 2 )
      /* could not zstat it, keep first info */
      st=0;
  }

  return st;
}
